<?php

namespace OrganizationsManagement\Controller;

use Exception;
use OrganizationsManagement\Traits\TResponse;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use OrganizationsManagement\Service\CollectionErrors as CollectionErrorsService;

class CollectionErrors
{

    use TResponse;

    /**
     * @var CollectionErrorsService
     */
    private $collectionErrorsService;

    /**
     * @param CollectionErrorsService
     */
    public function __construct(CollectionErrorsService $collectionErrorsService)
    {
        $this->collectionErrorsService = $collectionErrorsService;
    }

    public function find(Request $request, Response $response)
    {
        try {
            $parametros = $request->getParameters();
            $data = $this->collectionErrorsService->find($parametros);
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getMedidasVariaveis(Request $request)
    {
        try {
            $data = $this->collectionErrorsService->getMedidasVariaveis($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getElementosServOrca()
    {
        try {
            $data = $this->collectionErrorsService->getElementosServOrca();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getSeqContratos()
    {
        try {
            $data = $this->collectionErrorsService->getSeqContratos();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    } 

    public function getOrganizacaoParc()
    {
        try {
            $data = $this->collectionErrorsService->getOrganizacaoParc();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getCampoMedidaVariavel()
    {
        try {
            $data = $this->collectionErrorsService->getCampoMedidaVariavel();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    } 

    public function getCampoServOrca()
    {
        try {
            $data = $this->collectionErrorsService->getCampoServOrca();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
