<?php


namespace OrganizationsManagement\Controller;

use Exception;
use OrganizationsManagement\Traits\TResponse;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Util\InstanceProvider;
use \OrganizationsManagement\Service\SlaclienteAlert as SlaclienteAlertService;

class SlaclienteAlert
{
    use TResponse;

    private $slaclienteAlertService;

    public function __construct(SlaclienteAlertService $slaclienteAlertService){
        $this->slaclienteAlertService = $slaclienteAlertService;
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function findSlaclienteAlert(DTO\Request $request, DTO\Response $response){
        try{
            $cdcliente = $request->getParameter('CDCLIENTE');
            $cdgrupfase = $request->getParameter('CDGRUPFASE');
            $cdassunto = $request->getParameter('CDASSUNTO');
            $cdcomplexid = $request->getParameter('CDCOMPLEXID');
            $data = $this->slaclienteAlertService->findSlaclienteAlert($cdcliente, $cdgrupfase, $cdassunto, $cdcomplexid);
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function findStructureType(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $this->slaclienteAlertService->findStructureType();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function updateSlaclienteAlert(DTO\Request $request, DTO\Response $response)
    {
        try{
            $this->slaclienteAlertService->updateSlaclienteAlert($request->getParameters());
            $this->sendResponse([], 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }

    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function deleteSlaclienteAlert(DTO\Request $request, DTO\Response $response)
    {
        try{
            $this->slaclienteAlertService->deleteSlaclienteAlert($request->getParameters());
            $this->sendResponse([], 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function saveSlaclienteAlert(DTO\Request $request, DTO\Response $response)
    {
        try{
            $data = $request->getParameters();
            $idRespProdEstrut = $data["ID_RESP_PROD_ESTRUT"];
            $structureTypeId = $data["STRUCTURE_TYPE_ID"];
            $qthorasalert = $data["QTHORASALERT"];
            $idantesdepois = $data["IDANTESDEPOIS"];
            $cdcliente = $data["CDCLIENTE"];
            $cdassunto = $data["CDASSUNTO"];
            $cdgrupfase = $data["CDGRUPFASE"];
            $cdcomplexid = $data["CDCOMPLEXID"];
            $this->slaclienteAlertService->saveSlacliente($cdcliente, $cdassunto, $cdgrupfase, $cdcomplexid,
                                                     $idRespProdEstrut, $structureTypeId, $qthorasalert, $idantesdepois);
            $this->sendResponse([], 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
