<?php

namespace OrganizationsManagement\Service;

use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Repository\RedirecionamentoAssinatura as RedirecionamentoAssinaturaRepository;
use OrganizationsManagement\Util\InstanceProvider;

class RedirecionamentoAssinatura
{
    /**
     * @var EntityManager
     */
    private $entityManager;
    /**
     * @var RedirecionamentoAssinaturaRepository
     */
    private $redirecionamentoAssinaturaRepository;

    /**
     * Notice constructor.
     * @param EntityManager $entityManager
     * @param NoticeRepository $NoticeRepository
     */
    public function __construct(EntityManager $entityManager, RedirecionamentoAssinaturaRepository $redirecionamentoAssinaturaRepository)
    {
        $this->entityManager = $entityManager;
        $this->redirecionamentoAssinaturaRepository = $redirecionamentoAssinaturaRepository;
    }

    public function getParamCentrAssinatura()
    {
        return $this->redirecionamentoAssinaturaRepository->getParamCentrAssinatura();
    } 

    public function getTipoDocCertsign()
    {
        return $this->redirecionamentoAssinaturaRepository->getTipoDocCertsign();
    } 

    public function getRotasCentrAssinatura()
    {
        return $this->redirecionamentoAssinaturaRepository->getRotasCentrAssinatura();
    }

    public function getLogCentrAssinatura()
    {
        return $this->redirecionamentoAssinaturaRepository->getLogCentrAssinatura();
    }

    public function updateParamCentrAssinatura($data)
    {
        $environment = InstanceProvider::getLibEnvironment();
        $operador = $environment->getCurrentUserId();
        $nrorg = $environment->getCurrentOrganizationId();
        if ($data["ID"] != null) {
            $this->redirecionamentoAssinaturaRepository->updateParamCentrAssinatura($data, $operador, $nrorg);
        }
    }

    public function saveRotasAssinatura($rows)
    {
        $environment = InstanceProvider::getLibEnvironment();
        $operador = $environment->getCurrentUserId();
        $nrorg = $environment->getCurrentOrganizationId();

        foreach ($rows as $rota) {
            if ($rota["ID"] == 0) {
                $newCodeService = InstanceProvider::getService()->getNewCode();
                $newId = $newCodeService->getCodeByTableName('ROTAS_CENTR_ASSINATURA', $nrorg, 1, 10, true);
                $this->redirecionamentoAssinaturaRepository->insertRotaAssinatura($rota, $operador, $nrorg, $newId);
            } else {
                $this->redirecionamentoAssinaturaRepository->updateRotaAssinatura($rota, $operador, $nrorg);
            }
        }
    } 

    public function deleteRotaAssinatura($rows)
    {
        $this->redirecionamentoAssinaturaRepository->deleteRotaAssinatura($rows);
    }
}
