<?php

namespace OrganizationsManagement\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Teknisa\Entities;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Util;
use Teknisa\Model\Traits\TQuery;
use OrganizationsManagement\Util\InstanceProvider;
use Teknisa\Model\Geral\Repositories as GeralRepositories;
use Teknisa\Libs\Util\Environment as TeknisaEnvironment;
use Teknisa\Model\Geral\Entities\Organizacao as EntidadeOrganizacao;
use Teknisa\Model\Teknisa\Entities\OmParamOrg as EntidadeOmParamOrg;

class ScoreReport
{
    
    const TYPE_ISSUE        = Entities\CpCall::TYPE_ISSUE;
    const TYPE_CASO         = Entities\CpCall::TYPE_CASO;
    const TYPE_RICH_CLIENT  = Entities\CpCall::TYPE_RICH_CLIENT;
    const TYPE_TICKET       = Entities\CpCall::TYPE_TICKET;
    const CDOPER_DEFAULT      = '000000000997';
    const NRORG_DEFAUlT       = '20';
    const  NAME_COLUMNS_OUT_FILTER_QUERY_CALL = [
        "TEKNISA_ID" ,
        "STATUS_NAME" ,
        "PRODUCT_NAME" ,
        "NMOPERRESP" ,
        "UPDATED_AT" ,
        "CREATED_AT" ,
        "ALREADY_READ_NAME" ,
        "ALREADY_READ" ,
        "TASK_WORKITEM_NAME" ,
        "STRUCTURE_NAME" ,
        "VERSION" ,
        "REVISION" ,
    ];

    const  NAME_COLUMNS_FILTER_QUERY_CALL = [
        "REPORTED_BY" => "CALL.REPORTED_BY",
        "ID" => "CALL.NRCALL",
        "CALL_RELATIONSHIP" => "CALL.CALL_RELATIONSHIP",
        "CDOPERRESP" => "CCO.CDOPERADOR",
        "CDCLIENTE" => "CALL.CDCLIENTE",
        "PRODUCT_FILTER_ID" => "NVL(TO_CHAR(PRJ.ID),SUBSTR( PRO.CDARVPROD,length(PRO.CDARVPROD)-2,length(PRO.CDARVPROD))) ",
        "MODULES_ID" => "PRJ1.ID",
        "CREATED_AT_FILTER" => "CALL.DTINCLUSAO",
        "CREATED_AT" => "CALL.DTINCLUSAO",
        "CLASSIFICATION" => "CALL.CLASSIFICATION",
        "CDASSUNTO" => "CALL.CDASSUNTO",
        "UPDATED_AT_FILTER" => "CALL.UPDATED_CALL_AT",
        "UPDATED_AT" => "CALL.UPDATED_CALL_AT",
        "REPORTER_NAME" => "OPO.NMOPERADOR",
        "TASK_WORKITEM" => "CCW.CDITEMTRAB",
        "STRUCTURE_FILTER_ID" => "CALL.STRUCTURE",
        "CDFASETRAM" => "CALL.CDFASETRAM",
        "TASK_STATUS" => "CALL.TASK_STATUS",
        "STATUS" => "CALL.CALL_STATUS_CLI"
    ];

    const NAME_COLUMNS_NOT_SEARCH_IN = [
        "CALL_RELATIONSHIP",
        "REPORTED_BY",
        "CDOPERRESP",
        "MODULES_ID",
        "FASE_SOLI_PERIOD",
        "DIVIDE_SOLI",
        "FASE_SOLI_CDFASE",
        "FASE_SOLI_CDOPERADOR",
        "FASE_SOLI_CDOPERRESP",
        "FASE_SOLI_PERIOD",
        "FASE_SOLI_PERIOD_BOX",
        "CDCLIENTE",
        "PRODUCT_FILTER_ID",
        "CLASSIFICATION",
        "CDASSUNTO",
        "NRCALLLABEL",
        "STATUS",
        "CDFASETRAM",
        "TASK_STATUS",
        "CALL_WITHOUT_LABELS_CHECK",
        "UPDATED_AT_FILTER",
        "CREATED_AT_FILTER",
        "PRODUCT_FILTER_ID",
        "TASK_WORKITEM",
        "STRUCTURE_FILTER_ID",
        "ALREADY_READ",
        "actionForwardCall"
    ];

    const NAME_COLUMNS_FILTER_FASE_SOLI = [
        "FASE_SOLI_CDFASE",
        "FASE_SOLI_CDOPERADOR",
        "FASE_SOLI_CDOPERRESP",
        "FASE_SOLI_PERIOD",
        "FASE_SOLI_PERIOD_BOX",
    ];

    const NAME_COLUMNS_FILTER_FASE_SOLI_REQUIRED = [
        "FASE_SOLI_CDFASE",
        "FASE_SOLI_CDOPERADOR",
        "FASE_SOLI_CDOPERRESP",
    ];

    const  NAME_COLUMNS_FILTER_QUERY_CALL_ISSUE_CLOSED = [
        "REPORTED_BY" => "RER.EXTERNAL_CODE",
        "REFERENCE" => "TAS.NAME",
        "CALL_RELATIONSHIP" => "TAS.ID",
        "CDOPERRESP" => "REM.EXTERNAL_CODE",
        "CDCLIENTE" => "RES.EXTERNAL_CODE",
        "PRODUCT_FILTER_ID" => "TO_CHAR(TAS.PROJECT)",
        "MODULES_ID" => "TAS.PROJECT",
        "CREATED_AT_FILTER" => "TAS.CREATED_AT",
        "CREATED_AT" => "CALL.DTINCLUSAO",
        "CLASSIFICATION" => "TAS.CLASSIFICATION",
        "CDASSUNTO" => "CLA.CDASSUNTO",
        "UPDATED_AT_FILTER" => "TAS.UPDATED_AT",
        "UPDATED_AT" => "CALL.UPDATED_AT",
        "TASK_WORKITEM" => "TAW.CDITEMTRAB",
        "STRUCTURE_FILTER_ID" => "PRO.STRUCTURE",
        "CDFASETRAM" => "CALL.CDFASETRAM",
        "TASK_STATUS" => "TAS.TASK_STATUS"
    ];

    const  NAME_COLUMNS_FILTER_QUERY_CALL_SOLI_CLOSED = [
        "REPORTED_BY" => "OPE.CDOPERADOR",
        "REFERENCE" => "SER.DSREFESOLI",
        "CALL_RELATIONSHIP" => "SER.NRSOLISERV",
        "CDOPERRESP" => "SER.CDOPERRESP",
        "CDCLIENTE" => "CLI.CDCLIENTE",
        "PRODUCT_FILTER_ID" => "SUBSTR( PRO.CDARVPROD,length(PRO.CDARVPROD)-2,length(PRO.CDARVPROD))",
        "MODULES_ID" => "SER.CDPRODGRUP",
        "CREATED_AT_FILTER" => "SER.DTINCLUSAO",
        "CREATED_AT" => "CALL.DTINCLUSAO",
        "CLASSIFICATION" => "CLA.ID",
        "CDASSUNTO" => "SER.CDASSUNTO",
        "UPDATED_AT_FILTER" => "SER.DTINCLUSAO",
        "UPDATED_AT" => "CALL.UPDATED_AT",
        "TASK_WORKITEM" => "I.CDITEMTRAB",
        "STRUCTURE_FILTER_ID" => "POR.STRUCTURE",
        "CDFASETRAM" => "SER.CDFASETRAM",
        "TASK_STATUS" => "CALL.TASK_STATUS"
    ];

    const  NAME_COLUMNS_FILTER_QUERY_FASE_SOLI = [
        "FASE_SOLI_CDFASE" => "CDFASETRAM",
        "FASE_SOLI_CDOPERADOR" => "CDOPERADOR",
        "FASE_SOLI_CDOPERRESP" => "CDOPERRESP",
        "FASE_SOLI_PERIOD" => "DTALFASETRAM",
        "FASE_SOLI_PERIOD_BOX" => "PERIOD_BOX",
    ];

    protected $orderReverse = false;
    protected $orderBy = 'ID';

    const MAIN_TYPE_OF_PRODUCT_CALL = "P";
    const MAIN_TYPE_OF_SUPPORT_CALL = "S";

    /** @var EntityManager $entityManager */
    protected $entityManager;
    /** @var TeknisaEnvironment **/
    protected $environment;
    /** @var GeralRepositories\CpCall */
    protected $cpCallRepository;
    /** @var GeralRepositories\Operador  */
    private $repositoryOperador;
    /** @var \Teknisa\Model\Teknisa\Repositories\OmProduto  */
    private $omProdutoRepository;
    /** @var  \Teknisa\Model\Teknisa\Repositories\Organization*/
    private $editOrganization;
    /** @var FaturamentoRepository */
    private $faturamentoRepository;
    private $now;

    use TQuery;

    public function __construct(EntityManager $entityManager, TeknisaEnvironment $environment) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;

        $this->cpCallRepository = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'CpCall');
        $this->repositoryOperador = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Operador');
        $this->faturamentoRepository = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Parcnegocio');
        $this->omProdutoRepository = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProduto');
    }

    /**
     * @param $order
     * @return false|string
     */

    public function saveOrganizationFrame($data)
    {
        $data['IDPERCPIMP'] = null;
        $operador = Utilities::getCurrentUserId();
        $idaltnrscore = 'N';
        $entityManager = InstanceProvider::getEntityManager();

        /** @var \Teknisa\Model\Geral\Entities\Organizacao $organizacaoEntity */
        $organizacaoEntity = InstanceProvider::getRepository()->getOrganizacao()->findOneBy(array('nrorg' => $data['NRORG']));
        if($organizacaoEntity->getNrscore() != $data['NRSCORE']){
            $idaltnrscore = 'S';
        }
        $data['NRSCORE'] = InstanceProvider::getFactory()->getOrganizacao()->saveScoreOrganization($organizacaoEntity, $operador, $data, $idaltnrscore);
        $this->entityManager->persist($data['NRSCORE']);

        if($data['IDPERCPIMP']) {
            $data['IDPERCPIMP'] = $data['IDPERCPIMP'] == 'N' ? null : $data['IDPERCPIMP'];
            /** @var Entities\OmParamOrg $omParamOrgEntity */
            $omParamOrgEntity = InstanceProvider::getRepository()->getOmparamorg()->findOneBy(array('nrorg' => $data['NRORG']));
            $data['NRSCORE'] = InstanceProvider::getFactory()->getOmParamOrg()->saveIdpercpimp($omParamOrgEntity, $data);
            $this->entityManager->persist($data['NRSCORE']);
        }

        $this->entityManager->flush();
        return $data['NRSCORE']->getNrorg();
    }
    

    public function getAllSituationDividedByType($eliasSoliserv = 'SER')
    {
        $waitingTicket = Utilities::getWaitingTicket();
        $cdClienteBase = Utilities::getCdClienteBase();
        $allSituation  = Util\InstanceProvider::getRepository()->getOmParcnegocio()->getAllSituation($this->environment->getCurrentOrganizationId(),$waitingTicket);
        list($openedGep, $closedGep) = $this->getBeginAndEndCdfasetram();

        $waitingGEP = " (EXISTS (SELECT CDOPERADOR FROM OPERADOR WHERE CDCLIENTE <> '$cdClienteBase' AND CDOPERADOR = $eliasSoliserv.CDOPERRESP " ;

        $allSituationDividedByType = array(
            'GEP' => [
                'CLOSED' => " $eliasSoliserv.CDFASETRAM = $closedGep OR ($eliasSoliserv.IDSOLIENCE = 'S')",
                'OPENED' => " $eliasSoliserv.CDFASETRAM = $openedGep AND ($eliasSoliserv.IDSOLIENCE <> 'S')",
                'WAITING' => " $waitingGEP AND $eliasSoliserv.CDFASETRAM NOT IN ($closedGep) AND ($eliasSoliserv.IDSOLIENCE <> 'S'))) ",
                'IN_PROGRESS' => " $eliasSoliserv.CDFASETRAM NOT IN ($openedGep,$closedGep) AND ($eliasSoliserv.IDSOLIENCE <> 'S') ",
            ],
            'PM' => [
                'CLOSED' => [],
                'OPENED' => [],
                'WAITING' => [],
                'IN_PROGRESS' => [],
            ],
            'CRM' => [
                'CLOSED' => [],
                'OPENED' => [],
                'WAITING' => [],
                'IN_PROGRESS' => [],
            ],
        );
        foreach ($allSituation as $situation) {
            $allSituationDividedByType[$situation['PRODUCT']][$situation['SITUATION']][] = $situation['ID'];
        }
        return $allSituationDividedByType;
    }

    public function getBeginAndEndCdfasetram()
    {
        $paramgeral = $this->cpCallRepository->getBeginAndEndCdfasetram($this->environment->getCurrentOrganizationId());
        return array($paramgeral['CDFATRAMIWEB'],$paramgeral['CDFATRAMENCE']);
    }

    public function findAllOperadorRelationshipCliente($condition, $page = 0, $pageSize = 0){
        $cdCliente = Utilities::getCdClienteBase();
        $userId    = $this->environment->getCurrentUserId();
        $filter    = Utilities::createQueryFiltered($condition);
        $finalPos  = $pageSize*$page;
        $startPos  = $pageSize == 0 ? 0 : $finalPos-($pageSize-1);
        return $this->repositoryOperador->findAllOperadorRelationshipCliente($cdCliente, $userId, $filter, $startPos, $finalPos);
    }

    public function findAllOperadorResponsible($condition, $page, $pageSize){
        $filter    = Utilities::createQueryFiltered($condition);
        $filterTeknisa = "";
        $userId = $this->environment->getCurrentUserId();
        $teknisaId = Utilities::getCdClienteBase();

        if(!Utilities::currentUserIsTeknisa()){
            $filterTeknisa = "AND OP.CDCLIENTE IN (
            SELECT CDCLIENTE
            FROM(
                SELECT CLI.CDCLIENTE
                FROM OPERCLIE OCL
                JOIN OPERADOR OPE ON (OCL.CDOPERADOR  = OPE.CDOPERADOR AND OPE.CDOPERADOR = '". $userId ."')
                JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OCL.CDCLIENTE AND CLI.IDATIVO = 'S'))
                UNION
                (SELECT CLI.CDCLIENTE
                FROM OPERADOR OPE
                JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OPE.CDCLIENTE AND CLI.IDATIVO = 'S')
                WHERE OPE.CDOPERADOR = '". $userId ."')
                UNION
                SELECT '". $teknisaId ."' AS CDCLIENTE FROM DUAL
           ) ";
        }
        $finalPos  = $pageSize*$page;
        $startPos  = $pageSize == 0 ? 0 : $finalPos-($pageSize-1);
        return $this->repositoryOperador->findAllOperadorResponsible($filter, $filterTeknisa, $finalPos, $startPos);
    }

    /**
     * @param $conditions
     * @param $nrorg
     * @param $nrparcnegocio
     * @param $dtinicompet
     * @param $dtfincompet
     * @return array
     */
    public function faturamento($conditions, $nrparcnegocio, $dtinicompet, $dtfincompet, $page, $pageSize, $orderBy = null)
    {
        $limit = $page * $pageSize;

        $nrorg = $this->environment->getCurrentOrganizationId();
        $orderString = 'ORDER BY DTINICOMPET ASC, NRSEQCONTRATO';
        $offset = ($page - 1) * $pageSize;
        foreach($conditions as &$condition){
            if($condition["columnName"] == "DTFINCOMPET"){
                $condition["operator"] = "<=";
                $condition['value'] = "'" . $condition['value'] . "'";
            }
            if($condition["columnName"] == "DTINICOMPET"){
                $condition["operator"] = ">=";
                $condition['value'] = "'" . $condition['value'] . "'";
            }
        }
        $filter = Utilities::createQueryFiltered($conditions);
        if($orderBy){
            foreach($orderBy as $idx => $order) {
                $orderString = 'ORDER BY ' . $idx . ' ' . $order;
            }
        }
        $dtinicompetFilter = '';
        $dtfincompetFilter = '';
        $dtinicompetFilterMed = '';
        $dtfincompetFilterMed = '';
        if($dtinicompet){
            $dtinicompetFilter = "AND (TRUNC(TO_DATE('{$dtinicompet}', 'DD/MM/YYYY')) < TRUNC(FAT.DTINICOMPET) OR TRUNC(TO_DATE('{$dtinicompet}', 'DD/MM/YYYY')) < TRUNC(FAT.DTFINCOMPET))";
            $dtinicompetFilterMed = "AND (TRUNC(TO_DATE('{$dtinicompet}', 'DD/MM/YYYY')) < TRUNC(MED.DTINICOMPET) OR TRUNC(TO_DATE('{$dtinicompet}', 'DD/MM/YYYY')) < TRUNC(MED.DTFINCOMPET))";
        }
        if($dtfincompet){
            $dtfincompetFilter = "AND (TRUNC(TO_DATE('{$dtfincompet}', 'DD/MM/YYYY')) > TRUNC(FAT.DTINICOMPET) OR TRUNC(TO_DATE('{$dtfincompet}', 'DD/MM/YYYY')) > TRUNC(FAT.DTFINCOMPET))";
            $dtfincompetFilterMed = "AND (TRUNC(TO_DATE('{$dtfincompet}', 'DD/MM/YYYY')) > TRUNC(MED.DTINICOMPET) OR TRUNC(TO_DATE('{$dtfincompet}', 'DD/MM/YYYY')) > TRUNC(MED.DTFINCOMPET))";
        }
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getFaturamentoContrato($filter, $dtinicompetFilter, $dtfincompetFilter, $dtinicompetFilterMed, $dtfincompetFilterMed, $nrorg, $nrparcnegocio, $limit, $offset, $orderString);
    }

    public function getContratosForScore($nrparcnegocio) {
        $nrorg = $this->environment->getCurrentOrganizationId();
        $aditionalValues =  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getFinancialForScore($nrorg, $nrparcnegocio);
        $mainScore = Util\InstanceProvider::getRepository()->getOmParcnegocio()->getContratosForScore($nrorg, $nrparcnegocio);
        $mainScore[0]["EXPIRED"] = $aditionalValues[0]["EXPIRED"];
        $mainScore[0]["OPENED"] = $aditionalValues[0]["OPENED"];
        return $mainScore;

    }

    public function getItensContrato($nrfaturacont, $nrorg, $nrparcnegocio, $conditions){
        $filter = Utilities::createQueryFiltered($conditions);
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getItensContrato($nrfaturacont, $nrorg, $nrparcnegocio, $filter);
    }

    public function getItemsOrcamento($nrorg, $idadicitemorc, $nrseqcontrato, $nrseqorcamento){
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getItemsOrcamento($nrorg, $idadicitemorc, $nrseqcontrato, $nrseqorcamento);
    }

    public function getNegociacaoContratoCompleto($nrseqcontrato, $filters){
        $filter = Utilities::createQueryFiltered($filters);
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getNegociacaoContratoCompleto($nrseqcontrato, $filter);
    }

    public function getCompetenciaNegoCont($filters){
        $filter = Utilities::createQueryFiltered($filters);
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getCompetenciaNegoCont($filter);
    }

    public function getParcelanegocont($filters){
        $filter = Utilities::createQueryFiltered($filters);
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getParcelanegocont($filter);
    }

    public function getDescperiodonegocont_negociacaocontrato($filters){
        $filter = Utilities::createQueryFiltered($filters);
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getDescperiodonegocont_negociacaocontrato($filter);
    }

    public function getElements($nrorg, $idescolhido, $nrseqorcamento){
        if(!$nrseqorcamento){
            return false;
        }
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getElements($nrorg, $idescolhido, $nrseqorcamento);
    }

    public function getEscopoTIpoOrcamento(){
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getEscopoTIpoOrcamento();
    }

    public function getOrder($order)
    {
        $orderBy = '';
        if(empty($order)){
            $order = array('NRCONTRATOFCT.desc');
        }
        foreach ($order as $item) {
            $orderByItem = explode('.', $item);
            $orderBy .= ' ' . implode(" ", $orderByItem) . ',';
        }
        $orderBy = substr($orderBy, 0, -1);
        return 'ORDER BY '.$orderBy;
    }

    public function getContratos($dynamicFilter, $nrparcnegocio, $isFilter, $page, $pageSize, $orderBy, $search, $searchIn){
        $nrorg = $this->environment->getCurrentOrganizationId();
        $situacaoFilter = "";
        $orderString= "";
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $orderString = $this->getOrder($orderBy);
        if(!$isFilter){
            $situacaoFilter = "AND fct_contrato.idsituacaocontrato = 'A'";
        }
        $filter = '';
        $conditionsSearchQuery = '';
        if(!empty($search) && !empty($searchIn)){
            $conditionsSearchQuery = Util\InstanceProvider::getRepository()->getOmParcnegocio()->createSearch ($search, $searchIn);
        }
        if($dynamicFilter){ 
            $dynamicFilter = json_decode(utf8_encode(base64_decode($dynamicFilter)),true);
            $dynamicFilter2 = [];
            foreach ($dynamicFilter as $index => &$condition) {
                if($index == "NMTIPOORCAMENTO" || $index == "NRTIPOORCAMENTO"){
                    $dynamicFilter2['ESCOPO.' . $index] = $condition;
                }else{
                    $dynamicFilter2['fct_contrato.' . $index] = $condition;
                }
            }
        $filter = Util\InstanceProvider::getRepository()->getOmParcnegocio()->createDynamicFilter($dynamicFilter2, $dynamicFilter2, false);
        }
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getContratos($nrorg, $filter,$nrparcnegocio, $situacaoFilter, $limit, $offset, $orderString, $conditionsSearchQuery);
    }

    /**
     * @param $conditions
     * @param $nrparcnegocio
     * @return array
     */
    public function findProdutosContratados($nrparcnegocio)
    {
        return InstanceProvider::getRepository()->getLocalOrganization()->findProdutosContratados($nrparcnegocio);

    }

    public function getOrganizationScore(){
        $minRevenue = Utilities::getClassificationRevenueValueMinParameter();
        $organizationScore = InstanceProvider::getRepository()->getLocalOrganization()->getOrganizationScore($minRevenue);
        return $organizationScore;
    }

    public function obtemPorId($organizationId){
        $minRevenue = Utilities::getClassificationRevenueValueMinParameter();
        return InstanceProvider::getRepository()->getLocalOrganization()->obtemPorId($minRevenue, $organizationId);
    }

    public function findCallsByType($orgId)
    {
        $situationFilters = $this->getAllSituationDividedByType();

        $result  =  Util\InstanceProvider::getRepository()->getOmParcnegocio()->findCallsByType($situationFilters, $orgId);

        return $result;
    }

    public function findCalls($dynamicFilter,$search, $searchIn, $find, $order = array('UPDATED_AT.desc'), $page = 1, $limit = 50, $orgId)
    {
        $dynamicFilter = json_decode(utf8_encode(base64_decode($dynamicFilter)),true);
        $orderBy = $this->getOrderToCalls($order);
        
        list($dynamicFilter,$filterFaseSoli) = $this->getFilterFaseSoli($dynamicFilter,true);

        $allSituationDividedByType = $this->getAllSituationDividedByType();

        list($specialFilters, $dynamicFilter) = $this->createdSpecialFiltersClosed($dynamicFilter, $allSituationDividedByType);

        list($conditionsOutQuery, $dynamicFilter) = $this->getConditionsOutQuery($dynamicFilter);

        list($dynamicFilterIssue,$dynamicFilterSoli) = $this->parseGridColumnToQueryColumnClosed($dynamicFilter);

        $localCall =  InstanceProvider::getRepository()->getCpCallLocal();

        $conditionsSearchQuery = '';
        if(!empty($search) && !empty($searchIn)){
            $searchIn = $this->removeFilterNotSearchIn($searchIn);
            $conditionsSearchQuery = $localCall->createSearch ($search, $searchIn);
        }
        if(!empty($conditionsOutQuery)){
            $conditionsSearchQuery .= $localCall->createDynamicFilter($conditionsOutQuery, $conditionsSearchQuery,false);
        }

        $conditionsQueryIssue= '';
        if(!empty($dynamicFilterIssue)){
            $conditionsQueryIssue = $localCall->createDynamicFilter($dynamicFilterIssue, 'WHERE 1=1',false);
        }
        $conditionsQuerySoli= '';
        if(!empty($dynamicFilterSoli)){
            $conditionsQuerySoli = $localCall->createDynamicFilter($dynamicFilterSoli, 'WHERE 1=1',false);
        }
       

        return $localCall->findFastCallsClosed($conditionsQueryIssue, $conditionsQuerySoli,$conditionsSearchQuery,$filterFaseSoli,$orderBy, $page, $limit, $orgId, $allSituationDividedByType, $specialFilters);

    }

    public function removeFilterNotSearchIn($searchIn){
        $searchIn = array_diff($searchIn,self::NAME_COLUMNS_NOT_SEARCH_IN);
        return $searchIn;
    }

    public function situationFilter($situationTypes, $allSituationDividedByType, $aliasContato = 'CONT', $aliasTaskStattus = 'TST', $aliasSoliserv = 'SER')
    {
        
        $situationFilter = array(
            'ISSUE' =>'',
            'CONTATO' =>'',
            'SOLISERV' =>'',
        );
        if(!empty($situationTypes)){
            $issue = array();
            $contato = array();
            $soliserv  = array();
            asort($situationTypes);
            $isWaitingIssue= '';
            foreach ($situationTypes as $situationType) {
                if ($situationType == Entities\CpCall::CLOSE_SITUATION) {
                    $issue = $allSituationDividedByType['PM']['CLOSED'];
                    $contato = $allSituationDividedByType['CRM']['CLOSED'];
                    $soliserv[] = $allSituationDividedByType['GEP']['CLOSED'];
                } else if ($situationType == Entities\CpCall::OPEN_SITUATION) {
                    $issue = array_merge($issue,$allSituationDividedByType['PM']['OPENED']);
                    $contato = array_merge($contato,$allSituationDividedByType['CRM']['OPENED']);
                    $soliserv[] = $allSituationDividedByType['GEP']['OPENED'];
                } else  if ($situationType == Entities\CpCall::PROGRESS_SITUATION) {
                    $issue = array_merge($issue,$allSituationDividedByType['PM']['IN_PROGRESS']);
                    $contato = array_merge($contato,$allSituationDividedByType['CRM']['IN_PROGRESS']);
                    $soliserv[] = $allSituationDividedByType['GEP']['IN_PROGRESS'];
                } else if ($situationType == Entities\CpCall::WAITING_SPONSOR_SITUATION) {
                    $issue = array_merge($issue,$allSituationDividedByType['PM']['WAITING']);
                    $contato = array_merge($contato,$allSituationDividedByType['CRM']['WAITING']);
                    $soliserv[] = $allSituationDividedByType['GEP']['WAITING'];
                    $isWaitingIssue = true;
                }
            }
            $question = <<<SQL
                         'Q' = (SELECT NVL(CONTROL,'I') FROM (SELECT CTY.CONTROL
                                                FROM COMMENTS COM
                                                LEFT JOIN COMMENT_TYPE CTY ON(COM.COMMENT_TYPE_ID = CTY.ID )
                                                WHERE COM.TASK = TAS.ID
                                                ORDER BY COM.ID DESC)
                                                WHERE ROWNUM = 1 )
SQL;

            if(!empty($issue)){
                $contatoString = implode(',', $contato);
                $issueString = implode(',', $issue);
                $soliservString = implode('OR', $soliserv);
                $situationFilter['CONTATO'] = " AND $aliasContato.NRSEQSITUACAO IN($contatoString)";
                $situationFilter['SOLISERV'] = " AND ($soliservString)";

                if($isWaitingIssue){
                    $situationFilter['ISSUE'] = " AND ($aliasTaskStattus.ID IN($issueString) OR $question )";
                }else{
                    $situationFilter['SOLISERV'] .= "AND NOT ".$allSituationDividedByType['GEP']['WAITING'];
                    $situationFilter['ISSUE'] = " AND $aliasTaskStattus.ID IN($issueString)"." AND NOT ".$question;
                }
            }
        }
        return $situationFilter;
    }

    public function financeiro($nrparcnegocio, $dtemissao, $dtvencimento, $dtpagamento, $vencboleto, $dynamicFilter)
    {
        $emissaoFilter = "";
        $vencFilter = "";
        $pagFilter = "";
        if($dtemissao){
            $emissaoFilter = "  AND NF.DTEMISSAO BETWEEN '" . $dtemissao[0] . "' AND '" . $dtemissao[1] . "'";
        }

        if($dtvencimento){
            if($vencboleto){
                $vencFilter = "AND ((CR.DTATUAVENREC < TRUNC(SYSDATE)) AND (BO.IDSTATBREC IN ('A','E')) 
                OR (CR.DTATUAVENREC BETWEEN '" . $dtvencimento[0] . "' AND '" . $dtvencimento[1] . "'))";
            }else{
                $vencFilter =  "AND CR.DTATUAVENREC BETWEEN '" . $dtvencimento[0] . "' AND '" . $dtvencimento[1] . "'";
            }
        }
        if($dtpagamento){
            $pagFilter = " AND ((BR.DTBAIXAREC BETWEEN '". $dtpagamento[0] . "' AND '" . $dtpagamento[1] ."'))";
        }
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->scoreReportGetFinanceiro($nrparcnegocio, $emissaoFilter, $vencFilter, $pagFilter, $dynamicFilter);
    }

    public function getFilterFaseSoli($dynamicFilter,$demandClosed = false)
    {
        $localCall = InstanceProvider::getRepository()->getCpCallLocal();
        $filter = [];
        $conditionsQuery = '';
        $filterPeriodBox = false;
        $filterfase = false;
        foreach (self::NAME_COLUMNS_FILTER_FASE_SOLI as $filterName) {
            if (!empty($dynamicFilter[$filterName])) {
                if ($filterName == 'FASE_SOLI_PERIOD_BOX' && $dynamicFilter[$filterName] = 'Y') {
                    $filterPeriodBox = true;
                } else {
                    $filter[self::NAME_COLUMNS_FILTER_QUERY_FASE_SOLI[$filterName]] = $dynamicFilter[$filterName];
                }

                if(in_array($filterName,self::NAME_COLUMNS_FILTER_FASE_SOLI_REQUIRED)){
                    $filterfase = true;
                }

                unset($dynamicFilter[$filterName]);
            }
        }
        if ($filterfase) {
            $conditionsQuery = $localCall->createDynamicFilter($filter, 'WHERE 1=1', false);
            //$conditionsQuery = $this->filterFasePeriodBox($filterPeriodBox, $filter, $localCall, $conditionsQuery);
            $fieldQuery = "CALL.CALL_RELATIONSHIP";
            if($demandClosed){
                $fieldQuery = "SER.NRSOLISERV";
            } 
            $conditionsQuery = <<<SQL
            AND $fieldQuery IN (SELECT NRSOLISERV            
            FROM LOGFASETRAM            
            WHERE 1 = 1
            $conditionsQuery )
SQL;

        }
        return array($dynamicFilter,$conditionsQuery);
    }

    /**
     * @param array $arrayRevenues
     * @param EntidadeOrganizacao $org
     * @return array
     */
    private function setsInitautoUpdateNrscores(array $arrayRevenues, EntidadeOrganizacao $org)
    {
        $nrorg = $org->getNrorg();
        if (empty($arrayRevenues[$nrorg])) {
            $soma = 1;
            $allNrorg = array($nrorg);
            $revenue = 0.0;
        } else {
            $row = $arrayRevenues[$nrorg];
            $soma = empty($row['SOMA']) ? '1' : $row['SOMA'];
            $allNrorg = $row['ALL_NRORG'];
            $revenue = floatval($arrayRevenues[$nrorg]['FATURAMENTO']);
        }
        return array($allNrorg, $soma, $revenue);
    }

    private function getSituation($conditions, $allSituationDividedByType)
    {
        $situation = array(
            'ISSUE' =>'',
            'CONTATO' =>'',
            'SOLISERV' =>'',
        );
        foreach($conditions as $key => $filter) {
            if($filter['columnName'] == 'STATUS') {
                $situation = $this->situationFilter($conditions[$key]['value'], $allSituationDividedByType);
                unset($conditions[$key]);
            } 
        }
        return $situation;
    }

    /**
     * @param $dynamicFilter
     * @return mixed
     */
    private function parseGridColumnToQueryColumnClosed($dynamicFilter)
    {
        $dynamicFilterIssue = $dynamicFilter;
        $dynamicFilterSoli = $dynamicFilter;
        foreach (self::NAME_COLUMNS_FILTER_QUERY_CALL_ISSUE_CLOSED as $column => $columnQuery) {
            if (!empty($dynamicFilterIssue[$column])) {
                $dynamicFilterIssue[$columnQuery] = $dynamicFilterIssue[$column];
                unset($dynamicFilterIssue[$column]);
            }
        }
        foreach (self::NAME_COLUMNS_FILTER_QUERY_CALL_SOLI_CLOSED as $column => $columnQuery) {
            if (!empty($dynamicFilterSoli[$column])) {
                $dynamicFilterSoli[$columnQuery] = $dynamicFilterSoli[$column];
                unset($dynamicFilterSoli[$column]);
            }
        }
        return array($dynamicFilterIssue,$dynamicFilterSoli);
    }

    private function getConditionsOutQuery($dynamicFilter): array
    {
        $conditionsOutQuery = array();
        foreach (self::NAME_COLUMNS_OUT_FILTER_QUERY_CALL as $column) {
            if (!empty($dynamicFilter[$column])) {
                $conditionsOutQuery[$column] = $dynamicFilter[$column];
                unset($dynamicFilter[$column]);
            }
        }
        return array($conditionsOutQuery, $dynamicFilter);
    }


    private function createdSpecialFiltersClosed($dynamicFilter, $allSituationDividedByType): array
    {
        $specialFilters = '';

        if (!empty($dynamicFilter['CALL_WITHOUT_LABELS_CHECK'])) {
             unset($dynamicFilter['CALL_WITHOUT_LABELS_CHECK']);
        }
        if (!empty($dynamicFilter['NRCALLLABEL'])) {
            unset($dynamicFilter['NRCALLLABEL']);
        }
        if (!empty($dynamicFilter['STATUS'])) {
            $specialFilters = $this->situationFilter($dynamicFilter['STATUS'][0]['value'], $allSituationDividedByType);
            unset($dynamicFilter['STATUS']);
        }
        return array($specialFilters, $dynamicFilter);
    }


    /**
     * @param $allNrorg
     * @param $arrayRevenues
     * @return float
     */
    private function getTotalRevenues($allNrorg, $arrayRevenues)
    {
        $revenues = 0.0;
        foreach ($allNrorg as $nrorg) {
            if (!empty($arrayRevenues[$nrorg])) {
                $revenues += floatval($arrayRevenues[$nrorg]['FATURAMENTO']);
            }
        }
        return $revenues;
    }

    /**
     * @param array $arrayOmParamOrg
     * @param EntidadeOrganizacao $org
     * @param $revenues
     * @param \OrganizationsManagement\Factory\OmParamOrg $omParamOrgFactory
     * @param $revenue
     * @param \DateTime $now
     * @return mixed|EntidadeOmParamOrg
     */
    /**
     * @param $maintenanceType
     * @return array
     */
    private function getresultRevenuesIndexed($maintenanceType)
    {
        $resultRevenues = $this->omProdutoRepository->getCalculatedNrscoreAndRevenues($maintenanceType);
        $arrayRevenues = array();
        foreach ($resultRevenues as $fatu) {
            $fatu['ALL_NRORG'] = Utilities::setSelectMultiInQuery($fatu['ALL_NRORG']);
            $arrayRevenues[$fatu['NRORG']] = $fatu;
        }
        return $arrayRevenues;
    }

    private function getOrderToCalls($order)
    {
        $orderBy = '';
        if(empty($order)){
            $order = array('UPDATED_AT.desc');
        }
        foreach ($order as $item) {
            $orderByItem = explode('.', $item);
            if (!empty(self::NAME_COLUMNS_FILTER_QUERY_CALL[$orderByItem[0]])) {
                $orderByItem[0] = self::NAME_COLUMNS_FILTER_QUERY_CALL[$orderByItem[0]];
            }
            $orderBy .= ' ' . implode(" ", $orderByItem) . ',';
        }
        $orderBy = substr($orderBy, 0, -1);
        return $orderBy;
    }

    private function getAllFinancial()
    {
        $allFinancial = InstanceProvider::getRepository()->getOmParcnegocio()->getAllFinancialForScore();
        $arrayAllFinancial= array();
        foreach ($allFinancial as $fatu) {
            $arrayAllFinancial[$fatu['NRPARCNEGOCIO']] = $fatu;
        }
        return $arrayAllFinancial;
    }

    /**
     * @return EntidadeOmParamOrg[]
     */
    private function getOmParamOrgEntitiesIndexed()
    {
        /** @var EntidadeOmParamOrg[] $omParamOrgEntities */
        $omParamOrgEntities = InstanceProvider::getRepository()->getOmparamorg()->findBy(array());
        /** @var EntidadeOmParamOrg[] $arrayOmParamOrg */
        $arrayOmParamOrg = array();
        foreach ($omParamOrgEntities as $omParamOrgEntity) {
            $arrayOmParamOrg[$omParamOrgEntity->getNrorg()] = $omParamOrgEntity;
        }
        return $arrayOmParamOrg;
    }

    /**
     * @param $orderBy
     * @param $conditions
     * @return string
     */
    private function setOrderByInScore($orderBy, $conditions)
    {
        if (empty($orderBy)) {
            $conditions .= ' ORDER BY TO_NUMBER(NRSCORE) DESC';
        } else {
            foreach ($orderBy as $idx => $order) {
                if ($idx == 'VRCNPJFATU' || $idx == 'VRTOTALFATU') {
                    $idx = "TO_NUMBER($idx)";
                }
                $conditions .= ' ORDER BY ' . $idx . ' ' . $order;
            }
        }
        return $conditions;
    }

    /**
     * @param $isPrimary
     * @param $conditions
     * @return string
     */
    private function setFiltersAdditional($isPrimary, $conditions)
    {
        if ($isPrimary && $isPrimary == "S") {
            $conditions .= "AND NRORGPRINCIPAL IS NOT NULL";
        } else if ($isPrimary && $isPrimary == "P") {
            $conditions .= "AND NRORGPRINCIPAL IS NULL";
        }
        return $conditions;
    }

    public function getFindPadGer(){
        return InstanceProvider::getRepository()->getOperador()->getFindPadGer();
    }

}
