import { IDictionary, Metadata } from '@zeedhi/core';
import { LoadingService, Modal, ModalService, Select, SelectMultiple, Table, Text, Date, Form } from '@zeedhi/common';
import { Highlight } from '@zeedhi/zd-highlight-common';
import { OmDialogService } from '@/services/OmDialogService';
import Prism from 'prismjs';
import 'prismjs/components/prism-markup-templating.min';
import 'prismjs/components/prism-php.min';
import { TekGrid, TekGridColumn } from '@zeedhi/teknisa-components-common';
import moment from 'moment';
import { Utilities } from '@/util/Utilities';

export class AnaliseColetaController {
    public filterDate: any;

    public isVisibleGrid : boolean = true;

    public isVisibleHighlights = false;

    public modalMedidaVariavel!: Modal;

    public modalFilterAnaliseColeta!: Modal;

    public visibleGridMedidasVariaveis : boolean = true;

    public languageVar = 'php';

    private currentRow: any = {};

    private dadosFiltroAnaliseColeta: any = {};

    get frameAnaliseColetaIsVisible() {
        return !this.isVisibleGrid;
    }

    get frameDadosMedidaVariavel() {
        return !this.visibleGridMedidasVariaveis;
    }

    get getTituloModalMedidaVariavel() {
        return this.currentRow.CONTRATO;
    }

    public async createModalFilterAnaliseColeta() {
        const dtIniCompet = Utilities.firstDayOfCurrentMonth();
        const today = moment().format('DD/MM/YYYY');
        this.modalFilterAnaliseColeta = await ModalService.createFromJson('/analiseColeta/modal/modalFilterAnaliseColeta', true);
        await this.modalFilterAnaliseColeta.show();
        Metadata.getInstance<Date>('form-filtro-analise-coleta_DTINICOMPET').value = Utilities.formatDate(dtIniCompet);
        Metadata.getInstance<Date>('form-filtro-analise-coleta_DTFINCOMPET').value = today;
    }

    public destroyModalFilterAnaliseColeta() {
        this.modalFilterAnaliseColeta.destroy();
    }

    public async openModalFilterAnaliseColeta() {
        Metadata.getInstance<Date>('form-filtro-analise-coleta_DTINICOMPET').value = this.dadosFiltroAnaliseColeta.DTINICOMPET;
        Metadata.getInstance<Date>('form-filtro-analise-coleta_DTFINCOMPET').value = this.dadosFiltroAnaliseColeta.DTFINCOMPET;
        Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_NRPARCNEGOCIO').value = this.dadosFiltroAnaliseColeta.NRPARCNEGOCIO;
        Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_CDTPMEDIDASERV').value = this.dadosFiltroAnaliseColeta.CDTPMEDIDASERV;
        Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_IDELEMENTO').value = this.dadosFiltroAnaliseColeta.IDELEMENTO;
        Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_CDSERVORCA').value = this.dadosFiltroAnaliseColeta.CDSERVORCA;
        Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_IDTIPOORG').value = this.dadosFiltroAnaliseColeta.IDTIPOORG;
        Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_NRSEQCONTRATO').value = this.dadosFiltroAnaliseColeta.NRSEQCONTRATO;
        Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_IDSTATUS').value = this.dadosFiltroAnaliseColeta.IDSTATUS;
        Metadata.getInstance<Select>('form-filtro-analise-coleta_IDTPCOLETA').value = this.dadosFiltroAnaliseColeta.IDTPCOLETA;
        Metadata.getInstance<Select>('form-filtro-analise-coleta_IDSEMERRO').value = this.dadosFiltroAnaliseColeta.IDSEMERRO;
    }

    public closeModalFilterAnaliseColeta() {
        this.modalFilterAnaliseColeta.hide();
    }

    public async eventFilter({ event }: any) {
        event.preventDefault();
        await this.modalFilterAnaliseColeta.show();
        this.openModalFilterAnaliseColeta();
    }

    public limparCampos() {
        Metadata.getInstance<Form>('form-filtro-analise-coleta').clearForm();

        this.dadosFiltroAnaliseColeta = {
            NRPARCNEGOCIO: null,
            CDTPMEDIDASERV: null,
            IDELEMENTO: null,
            CDSERVORCA: null,
            IDTIPOORG: null,
            NRSEQCONTRATO: null,
            IDSTATUS: null,
            IDTPCOLETA: null,
            IDSEMERRO: null,
        };
    }

    public filtrarElemento() {
        const grid = Metadata.getInstance<TekGrid>('analise-coleta-grid');
        const form = Metadata.getInstance<Form>('form-filtro-analise-coleta');

        this.dadosFiltroAnaliseColeta.DTINICOMPET = Metadata.getInstance<Date>('form-filtro-analise-coleta_DTINICOMPET').value;
        this.dadosFiltroAnaliseColeta.DTFINCOMPET = Metadata.getInstance<Date>('form-filtro-analise-coleta_DTFINCOMPET').value;
        this.dadosFiltroAnaliseColeta.NRPARCNEGOCIO = Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_NRPARCNEGOCIO').value;
        this.dadosFiltroAnaliseColeta.CDTPMEDIDASERV = Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_CDTPMEDIDASERV').value;
        this.dadosFiltroAnaliseColeta.IDELEMENTO = Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_IDELEMENTO').value;
        this.dadosFiltroAnaliseColeta.CDSERVORCA = Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_CDSERVORCA').value;
        this.dadosFiltroAnaliseColeta.IDTIPOORG = Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_IDTIPOORG').value;
        this.dadosFiltroAnaliseColeta.NRSEQCONTRATO = Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_NRSEQCONTRATO').value;
        this.dadosFiltroAnaliseColeta.IDSTATUS = Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_IDSTATUS').value;
        this.dadosFiltroAnaliseColeta.IDTPCOLETA = Metadata.getInstance<Select>('form-filtro-analise-coleta_IDTPCOLETA').value;
        this.dadosFiltroAnaliseColeta.IDSEMERRO = Metadata.getInstance<Select>('form-filtro-analise-coleta_IDSEMERRO').value;

        if (Utilities.validateForm(form)) {
            grid.datasource.setFilter({
                DTINICOMPET: Metadata.getInstance<Date>('form-filtro-analise-coleta_DTINICOMPET').value,
                DTFINCOMPET: Metadata.getInstance<Date>('form-filtro-analise-coleta_DTFINCOMPET').value,
                NRPARCNEGOCIO: Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_NRPARCNEGOCIO').value,
                CDTPMEDIDASERV: Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_CDTPMEDIDASERV').value,
                IDELEMENTO: Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_IDELEMENTO').value,
                CDSERVORCA: Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_CDSERVORCA').value,
                IDTIPOORG: Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_IDTIPOORG').value,
                NRSEQCONTRATO: Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_NRSEQCONTRATO').value,
                IDSTATUS: Metadata.getInstance<SelectMultiple>('form-filtro-analise-coleta_IDSTATUS').value,
                IDSEMERRO: Metadata.getInstance<Select>('form-filtro-analise-coleta_IDSEMERRO').value,
                IDTPCOLETA: Metadata.getInstance<Select>('form-filtro-analise-coleta_IDTPCOLETA').value,
            });
            this.modalFilterAnaliseColeta.hide();
        }
    }

    public get addDatasourceScript() {
        const queries = Metadata.getInstances('SCRIPT');
        return queries.find((query: any) => query.parent.name === 'fifithRowHighlightScript');
    }

    public get organizationHeader() {
        const queries = Metadata.getInstances('ORGANIZATION_HEADER');
        return queries.find((query: any) => query.parent.name === 'ZdOrganizationHeader');
    }

    public get addDatasourceScriptCount() {
        const queries = Metadata.getInstances('SCRIPT_COUNT');
        return queries.find((query: any) => query.parent.name === 'fifithRowScriptCount');
    }

    public get addDatasourcePrepStatus() {
        const queries = Metadata.getInstances('PREPSTATUS');
        return queries.find((query: any) => query.parent.name === 'firstRowStatus');
    }

    public get addDatasourceIdTipoOrg() {
        const queries = Metadata.getInstances('IDTIPOORG');
        return queries.find((query: any) => query.parent.name === 'firstRowIdTipoOrg');
    }

    public get addDatasourceIsPrimary() {
        const queries = Metadata.getInstances('IS_PRIMARY');
        return queries.find((query: any) => query.parent.name === 'firstRowIdPrimaria');
    }

    public get addDatasourceNmOrgPrincipal() {
        const queries = Metadata.getInstances('NMORGPRINCIPAL');
        return queries.find((query: any) => query.parent.name === 'firstRowOrganizationPrimary');
    }

    public get addDatasourceOrganization() {
        const queries = Metadata.getInstances('NMPARCEIRO');
        return queries.find((query: any) => query.parent.name === 'firstRowOrganization');
    }

    public get addDatasourceProduct() {
        const queries = Metadata.getInstances('PRODUCT');
        return queries.find((query: any) => query.parent.name === 'firstRowProduct');
    }

    public get addDatasourceVariableMeasure() {
        const queries = Metadata.getInstances('VARIABLE_MEASURE');
        return queries.find((query: any) => query.parent.name === 'FirstRowVariableMeasure');
    }

    public get addDatasourceService() {
        const queries = Metadata.getInstances('SERVICE');
        return queries.find((query: any) => query.parent.name === 'SecondRowService');
    }

    public get addDatasourceContractCode() {
        const queries = Metadata.getInstances('CONTRACT_CODE');
        return queries.find((query: any) => query.parent.name === 'SecondRowContractCode');
    }

    public get addDatasourceData() {
        const queries = Metadata.getInstances('DATA');
        return queries.find((query: any) => query.parent.name === 'SecondRowData');
    }

    public get addDatasourceLastSuccessfulCollect() {
        const queries = Metadata.getInstances('LAST_SUC_COLLECT');
        return queries.find((query: any) => query.parent.name === 'SecondRowDataLastSucCollect');
    }

    public get addDatasourceStringCon() {
        const queries = Metadata.getInstances('DSSTRINGCONEXAO');
        return queries.find((query: any) => query.parent.name === 'ThirdRowStringCon');
    }

    public get addDatasourceWallet() {
        const queries = Metadata.getInstances('IDUTWALLET');
        return queries.find((query: any) => query.parent.name === 'ThirdRowWallet');
    }

    public get addDatasourceCollector() {
        const queries = Metadata.getInstances('IDCOLETOR');
        return queries.find((query: any) => query.parent.name === 'ThirdRowCollector');
    }

    public get addDatasourceTpColeta() {
        const queries = Metadata.getInstances('IDTPCOLETA');
        return queries.find((query: any) => query.parent.name === 'ThirdRowTpColeta');
    }

    public get addDatasourceDtIniCompetencia() {
        const queries = Metadata.getInstances('DTINICOMPET');
        return queries.find((query: any) => query.parent.name === 'ThirdTemp1RowDtIniCompet');
    }

    public get addDatasourceDtFinCompetencia() {
        const queries = Metadata.getInstances('DTFINCOMPET');
        return queries.find((query: any) => query.parent.name === 'ThirdTemp1RowDtFinCompet');
    }

    public get addDatasourceQtContratual() {
        const queries = Metadata.getInstances('QTCONTRATUAL');
        return queries.find((query: any) => query.parent.name === 'ThirdTemp1RowQtContratual');
    }

    public get addDatasourceQtMedida() {
        const queries = Metadata.getInstances('QTMEDIDA');
        return queries.find((query: any) => query.parent.name === 'ThirdTemp1RowQtMedida');
    }

    public get addDatasourceQtFaturada() {
        const queries = Metadata.getInstances('QTFATURADA');
        return queries.find((query: any) => query.parent.name === 'ThirdTemp1RowQtFaturada');
    }

    public get addDatasourceDsObsColeta() {
        const queries = Metadata.getInstances('DSOBSCOLETA');
        return queries.find((query: any) => query.parent.name === 'fourthRowDsObsColeta');
    }

    public get addDatasourceIdStatus() {
        const queries = Metadata.getInstances('IDSTATUS');
        return queries.find((query: any) => query.parent.name === 'fourthRowStatus');
    }

    public get addDatasourceCollapseCardScriptCountPHP() {
        const queries = Metadata.getInstances('SCRIPT_COUNT_PHP');
        return queries.find((query: any) => query.parent.name === 'collapseCardScriptCountPHP');
    }

    public get addDatasourceCollapseCardScriptPHP() {
        const queries = Metadata.getInstances('SCRIPT_PHP');
        return queries.find((query: any) => query.parent.name === 'collapseCardHighlightScriptPHP');
    }

    public get addDatasourceCollapseCardScriptCountSQL() {
        const queries = Metadata.getInstances('SCRIPT_COUNT_SQL');
        return queries.find((query: any) => query.parent.name === 'collapseCardScriptCountSQL');
    }

    public get addDatasourceCollapseCardScriptSQL() {
        const queries = Metadata.getInstances('SCRIPT_SQL');
        return queries.find((query: any) => query.parent.name === 'collapseCardHighlightScriptSQL');
    }

    public getPrimaryValue(value: string) {
        return value === 'P' ? 'Primária' : 'Secundária';
    }

    public getIdTipoOrgValue(value: string) {
        if (value === 'S') return 'SaaS';
        if (value === 'H') return 'Hosting';
        return 'Local';
    }

    public getStatusValue(value: string) {
        const allStatus = {
            P: 'Base Preparada',
            D: 'Base Bloqueada',
            Z: 'Base Preparada (Zmart Incluso)',
            C: 'Gerar Estrutura',
            Q: 'Gerar Dados Iniciais',
            S: 'Geração de Estrutura Solicitada',
            I: 'Organização Inativada',
            R: 'Organização Inativada - Base Removida',
            N: 'Preparação Pendente (Local/Hosting)',
            L: 'Organização Local/Hosting Preparada',
            Y: 'Organização Local/Hosting Preparada (Zmart Incluso)',
            J: 'Organização Local/Hosting Inativada',
            O: 'Preparação Pendente (Secundária)',
            M: 'Organização Secundária Preparada',
            X: 'Organização Secundária Preparada (Zmart Incluso)',
            K: 'Organização Secundária Inativada',
        };
        return value ? allStatus[value] : '';
    }

    public getIdstatus(value: string) {
        const allStatus = {
            E: 'Erro (Coleta Automática)',
            W: 'Alerta (Parametrização)',
            A: 'Alerta (Coleta Automática)',
            O: 'Coleta OK',
            X: 'Coleta OK (Erros em coletas Posteriores)',
        };
        return value ? allStatus[value] : '';
    }

    public async organizationGridRowClick(params) {
        this.isVisibleGrid = false;
        const fifthTable = Metadata.getInstance<Table>('fifthRow');
        const thirdTable = Metadata.getInstance<Table>('thirdRow');
        const lastSucCollect = Metadata.getInstance<Table>('SecondRowDataLastSucCollect');
        const SQLCollapseCard = Metadata.getInstance<Table>('SQL_Sub_Scripts');
        const PHPCollapseCard = Metadata.getInstance<Table>('PHP_Sub_Scripts');
        try {
            LoadingService.show();
            this.addDatasourcePrepStatus.text = this.getStatusValue(params.row.PREPSTATUS);
            this.addDatasourceIdTipoOrg.text = this.getIdTipoOrgValue(params.row.IDTIPOORG);
            this.addDatasourceIsPrimary.text = this.getPrimaryValue(params.row.IS_PRIMARY);
            this.addDatasourceIdStatus.text = this.getIdstatus(params.row.IDSTATUS);
            this.addDatasourceNmOrgPrincipal.text = params.row.NMORGPRINCIPAL;
            this.addDatasourceOrganization.text = params.row.NMPARCEIRO;
            this.organizationHeader.text = params.row.NMPARCEIRO;
            this.addDatasourceVariableMeasure.text = params.row.DSTPMEDIDASERV;
            this.addDatasourceProduct.text = params.row.NMELEMENTO;
            this.addDatasourceService.text = params.row.NMSERVORCA;
            this.addDatasourceContractCode.text = params.row.CONTRATO;
            this.addDatasourceData.text = params.row.DTINCLUSAO;
            this.addDatasourceLastSuccessfulCollect.text = params.row.DTULTCOLETASUC;
            this.addDatasourceDsObsColeta.text = params.row.DSOBSCOLETA;
            this.addDatasourceDtIniCompetencia.text = params.row.DTINICOMPET;
            this.addDatasourceDtFinCompetencia.text = params.row.DTFINCOMPET;
            this.addDatasourceQtContratual.text = params.row.QTCONTRATUAL;
            this.addDatasourceQtMedida.text = params.row.QTMEDIDA;
            this.addDatasourceQtFaturada.text = params.row.QTFATURADA;
            this.addDatasourceStringCon.text = params.row.DSSTRINGCONEXAO;
            this.addDatasourceCollector.text = params.row.IDCOLETOR;
            this.addDatasourceTpColeta.text = params.row.IDTPCOLETA;
            this.addDatasourceScriptCount.code = params.row.SCRIPT_COUNT;
            this.addDatasourceScript.code = params.row.SCRIPT_LOG;
            this.addDatasourceWallet.text = params.row.IDUTWALLET;
            thirdTable.isVisible = true;
            if (params.row.IDUTWALLET === 'S') {
                this.addDatasourceWallet.text = 'Sim';
            } else {
                this.addDatasourceWallet.text = 'Não';
            }
            if (params.row.IDTPERRO === 'E' || params.row.IDTPERRO === 'A') {
                this.addDatasourceStringCon.text = params.row.DSSTRINGCONEXAO;
                this.addDatasourceCollector.text = params.row.IDCOLETOR;
                this.addDatasourceTpColeta.text = params.row.IDTPCOLETA;
                this.addDatasourceScriptCount.code = params.row.SCRIPT_COUNT;
                this.addDatasourceScript.code = params.row.SCRIPT_LOG;
                this.addDatasourceWallet.text = params.row.IDUTWALLET;
                if (params.row.IDUTWALLET === 'S') {
                    this.addDatasourceWallet.text = 'Sim';
                } else {
                    this.addDatasourceWallet.text = 'Não';
                }
                fifthTable.isVisible = true;
                thirdTable.isVisible = true;
                lastSucCollect.isVisible = true;
                PHPCollapseCard.isVisible = false;
                SQLCollapseCard.isVisible = false;
                this.isVisibleHighlights = false;
                if (params.row.IDTPSCRIPT === 'SQL') {
                    this.addDatasourceScript.language = 'sql';
                } else {
                    this.addDatasourceScript.language = 'php';
                }
            } else {
                this.addDatasourceCollapseCardScriptCountPHP.code = params.row.SCRIPT_COUNT_PHP;
                this.addDatasourceCollapseCardScriptPHP.code = params.row.SCRIPT_LOG_PHP;
                this.addDatasourceCollapseCardScriptCountSQL.code = params.row.SCRIPT_COUNT_SQL;
                this.addDatasourceCollapseCardScriptSQL.code = params.row.SCRIPT_LOG_SQL;
                fifthTable.isVisible = false;
                thirdTable.isVisible = true;
                lastSucCollect.isVisible = false;
                PHPCollapseCard.isVisible = true;
                SQLCollapseCard.isVisible = true;
                this.isVisibleHighlights = true;
            }
            LoadingService.hide();
        } catch (e) {
            LoadingService.hide();
            OmDialogService.showErrorBackend(e);
        }
    }

    get isVisibleHighLights(): boolean {
        return this.isVisibleHighlights;
    }

    get isVisibleDbVersionCard(): boolean {
        return !this.isVisibleGrid;
    }

    public closeButton() {
        this.isVisibleGrid = true;
    }

    public loadLanguage = this.loadLanguageAux.bind(this);

    public loadLanguageAux() {
        const query = this.addDatasourceScript;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCollapseCardScriptPHP = this.loadLanguageAuxCollapseCardScriptPHP.bind(this);

    public loadLanguageAuxCollapseCardScriptPHP() {
        const query = this.addDatasourceCollapseCardScriptPHP;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCollapseCardCountPHP = this.loadLanguageAuxCollapseCardCountPHP.bind(this);

    public loadLanguageAuxCollapseCardCountPHP() {
        const query = this.addDatasourceCollapseCardScriptCountPHP;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCollapseCardScriptSQL = this.loadLanguageAuxCollapseCardScriptSQL.bind(this);

    public loadLanguageAuxCollapseCardScriptSQL() {
        const query = this.addDatasourceCollapseCardScriptSQL;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCollapseCardCountSQL = this.loadLanguageAuxCollapseCardCountSQL.bind(this);

    public loadLanguageAuxCollapseCardCountSQL() {
        const query = this.addDatasourceCollapseCardScriptCountSQL;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCount = this.loadLanguageAuxCount.bind(this);

    public loadLanguageAuxCount() {
        const query = this.addDatasourceScriptCount;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageExpression() {
        const { code, language } = Metadata.getInstance<Highlight>('expression');
        return Prism.highlight(code, Prism.languages[language], language);
    }

    public rowStyleConditions(row: IDictionary) {
        if (row.IDSTATUS === 'O') {
            return {
                cssClass: '',
            };
        }

        if ((row.IDSTATUS === 'W') || (row.IDSTATUS === 'A')) {
            return {
                cssClass: 'idstatusWarning',
            };
        }

        if (row.IDSTATUS === 'X') {
            return {
                cssClass: 'idstatusLaterErrors',
            };
        }

        if (row.IDSTATUS === 'E') {
            return {
                cssClass: 'idstatusErro',
            };
        }

        return { cssClass: '' };
    }

    public async criarModalMedidaVariavel() {
        this.modalMedidaVariavel = await ModalService.createFromJson('/analiseColeta/modal/modalMedidaVariavel', true);
    }

    public destruirModal() {
        this.modalMedidaVariavel?.destroy();
    }

    public fecharModalFilter() {
        this.visibleGridMedidasVariaveis = true;
        this.modalMedidaVariavel.hide();
    }

    public backGridMedidaVariavel() {
        this.visibleGridMedidasVariaveis = true;
    }

    public getMedidasVariaveis() {
        const grid = Metadata.getInstance<TekGrid>('gridMedidasVariaveis');
        const conditions = [
            { columnName: 'NRPRODUTO_ID', operator: '=', value: (this.currentRow.IDELEMENTORAIZ !== null) ? this.currentRow.IDELEMENTORAIZ : null },
            { columnName: 'CDSERVORCA', operator: '=', value: this.currentRow.CDSERVORCA },
            { columnName: 'initCompetence', operator: '=', value: this.currentRow.DTINICOMPET },
            { columnName: 'endCompetence', operator: '=', value: this.currentRow.DTFINCOMPET },
            { columnName: 'contractId', operator: '=', value: this.currentRow.NRSEQCONTRATO },
            { columnName: 'qtFatura', operator: '=', value: this.currentRow.QTFATURADA },
            { columnName: 'nrparcnegocio', operator: '=', value: this.currentRow.NRPARCNEGOCIO },
        ];
        grid.datasource.setFilter(conditions);
    }

    public async exibirModalFilter() {
        const grid = Metadata.getInstance<TekGrid>('analise-coleta-grid');
        await this.modalMedidaVariavel.show();
        this.currentRow = grid.datasource.currentRow;
        this.getMedidasVariaveis();
    }

    public clickRowMedidasVariaveis({ row }: any) {
        this.visibleGridMedidasVariaveis = false;
        setTimeout(() => {
            this.popularModalMedidasVariaveis(row);
            this.popularModalGridMedidaVariavel(row);
        }, 800);
    }

    public popularModalMedidasVariaveis(row: any) {
        Metadata.getInstance<Text>('medidaVariavelText').text = row.measureName;
        Metadata.getInstance<Text>('qtdMinimaText').text = row.countMin;
        Metadata.getInstance<Text>('qtdfaturaText').text = row.qtFatura;
        Metadata.getInstance<Text>('qtdFaturamentoText').text = row.count;
        Metadata.getInstance<Text>('dataColetaText').text = row.date;
        Metadata.getInstance<Text>('inicioVigenciaText').text = row.initCompetence;
        Metadata.getInstance<Text>('finalVigenciaText').text = row.endCompetence;
    }

    public popularModalGridMedidaVariavel(row: any) {
        const grid = Metadata.getInstance<TekGrid>('gridDadosColetados');
        if (row.log) {
            this.modularGridmodal(row.log, grid);
            grid.datasource.updateData(row.log);
        }
    }

    public modularGridmodal(row: any, grid: any) {
        if (row[0] !== undefined) {
            const rows = row[0];
            grid.columns = Object.keys(rows).map((key) => new TekGridColumn({ name: key, label: key }, grid));
        }
    }
}
